/*================================================================================================*
 *   LTR-
 *   TCP/IP ,    IP-
 *
 * ltriptest list        -   
 * ltriptest add <ip>    -  IP  (   INI )
 * ltriptest del <ip>    -  IP  (   INI )
 * ltriptest conn <ip>   -   
 * ltriptest disc <ip>   -   
 *================================================================================================*/

#include <stdio.h>
#include <windows.h>
#include "ltr\\include\\ltrapidefine.h"
#include "ltr\\include\\ltrapi.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void oem_printf
    (
    const char *Format,
    ...
    )
    {
    char str[2048];

    va_list va;
    va_start(va, Format);
    vsnprintf(str, sizeof(str) - 1, Format, va);
    va_end(va);
    str[sizeof(str) - 1] = '\0';
    CharToOemBuff(str, str, strlen(str));
    printf(str);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ListCrates(TLTR* pltr, DWORD dummy)
    {
    TIPCRATE_ENTRY* info;
    const DWORD ip_net =  0;
    const DWORD ip_mask = 0;
    DWORD num_found, num_read;
    DWORD i;
    INT ltr_result;

    //    
    ltr_result = LTR_GetListOfIPCrates(pltr, 0, ip_net, ip_mask, &num_found, NULL, NULL);
    if (ltr_result != LTR_OK)
        {
        oem_printf("    IP : %s\n", LTR_GetErrorString(ltr_result));
        return;
        }

    info = malloc(sizeof(TIPCRATE_ENTRY) * num_found);
    if (!info)
        {
        oem_printf(" \n");
        return;
        }

    ltr_result = LTR_GetListOfIPCrates(pltr, num_found, ip_net, ip_mask, NULL, &num_read, info);
    if (ltr_result != LTR_OK)
        {
        oem_printf("    IP : %s\n", LTR_GetErrorString(ltr_result));
        free(info);
        return;
        }

    if (!num_read)
        {
        oem_printf("IP- \n");
        }
    else
        {
        oem_printf(" IP : %u\n", num_found);
        if (num_read != num_found)
            oem_printf("!     %u \n", num_read);
        oem_printf("\nIP                           No.\n");

        for (i = 0; i < num_read; i++)
            {
            char* stst;
            struct in_addr addr;
            switch (info[i].status)
                {
                case CRATE_IP_STATUS_OFFLINE:    stst = "offline   "; break;
                case CRATE_IP_STATUS_CONNECTING: stst = "connecting"; break;
                case CRATE_IP_STATUS_ONLINE:     stst = "online    "; break;
                case CRATE_IP_STATUS_ERROR:      stst = "error     "; break;
                default:                         stst = "invalid   "; break;
                }
            addr.s_addr = htonl(info[i].ip_addr);
            oem_printf("%15s  %s  %s   %s  %s\n",
                inet_ntoa(addr),
                info[i].is_dynamic ? "dynamic" : "static ",
                (info[i].flags & CRATE_IP_FLAG_AUTOCONNECT) ? "yes" : "no ",
                stst,
                info[i].serial_number);
            }
        }
    free(info);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void AddCrate(TLTR* pltr, DWORD ip)
    {
    if (!ip)
        {
        oem_printf("   IP-\n");
        return;
        }
    oem_printf("  %d.%d.%d.%d ...",
        (ip >> 24) & 255, (ip >> 16) & 255, (ip >> 8) & 255, ip & 255);
    if (LTR_AddIPCrate(pltr, ip, 0, TRUE) == LTR_OK)
        {
        oem_printf(" ok\n");
        }
    else
        {
        oem_printf(" !\n");
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void DeleteCrate(TLTR* pltr, DWORD ip)
    {
    if (!ip)
        {
        oem_printf("   IP-\n");
        return;
        }
    oem_printf("  %d.%d.%d.%d ...",
        (ip >> 24) & 255, (ip >> 16) & 255, (ip >> 8) & 255, ip & 255);
    if (LTR_DeleteIPCrate(pltr, ip, TRUE) == LTR_OK)
        {
        oem_printf(" ok\n");
        }
    else
        {
        oem_printf(" !\n");
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ConnectCrate(TLTR* pltr, DWORD ip)
    {
    if (!ip)
        {
        oem_printf("   IP-\n");
        return;
        }
    oem_printf("  %d.%d.%d.%d ...",
        (ip >> 24) & 255, (ip >> 16) & 255, (ip >> 8) & 255, ip & 255);
    if (LTR_ConnectIPCrate(pltr, ip) == LTR_OK)
        {
        /*  . */
        /*    10        */
        BOOL done = FALSE;
        int count = 0;
        TIPCRATE_ENTRY info;
        DWORD n;
        while (!done && (count < 10))
            {
            if (count++) Sleep(1000);
            if (LTR_GetListOfIPCrates(pltr, 1, ip, 0xFFFFFFFF, NULL, &n, &info) != LTR_OK)
                break;
            if ((n > 0) && (info.status == CRATE_IP_STATUS_ONLINE))
                done = TRUE;
            }
        if (done)
            oem_printf(" ok\n");
        else
            oem_printf("   \n");
        }
    else
        {
        oem_printf(" !\n");
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void DisconnectCrate(TLTR* pltr, DWORD ip)
    {
    if (!ip)
        {
        oem_printf("   IP-\n");
        return;
        }
    oem_printf("  %d.%d.%d.%d ...",
        (ip >> 24) & 255, (ip >> 16) & 255, (ip >> 8) & 255, ip & 255);
    if (LTR_DisconnectIPCrate(pltr, ip) == LTR_OK)
        {
        /*  . */
        /*    10        */
        BOOL done = FALSE;
        int count = 0;
        TIPCRATE_ENTRY info;
        DWORD n;
        while (!done && (count < 10))
            {
            if (count++) Sleep(1000);
            if (LTR_GetListOfIPCrates(pltr, 1, ip, 0xFFFFFFFF, NULL, &n, &info) != LTR_OK)
                break;
            if ((n > 0) && (info.status == CRATE_IP_STATUS_OFFLINE))
                done = TRUE;
            }
        if (done)
            oem_printf(" ok\n");
        else
            oem_printf("   \n");
        }
    else
        {
        oem_printf(" !\n");
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int main(int argc, char *argv[])
    {
    TLTR ltr;
    int ltr_result;
    DWORD ip_addr = 0;
    void (*func)(TLTR*, DWORD);

    oem_printf("*   LTR- *\n");

    /*   */
    if (argc > 2)
        {
        ip_addr = inet_addr(argv[2]);
        if ((ip_addr == INADDR_NONE) || (ip_addr == INADDR_ANY))
            {
            ip_addr = 0;
            }
        else
            {
            ip_addr = ntohl(ip_addr);
            }
        }
    if ((argc < 2) || !strcmp(argv[1], "-h") || strchr(argv[1], '?'))
        {
        oem_printf(" :\n"
            " ltriptest list        -   \n"
            " ltriptest add <ip>    -  IP  (   INI )\n"
            " ltriptest del <ip>    -  IP  (   INI )\n"
            " ltriptest conn <ip>   -   \n"
            " ltriptest disc <ip>   -   \n"
            );
        exit(0);
        }

    if (!strcmp(argv[1], "list"))
        func = ListCrates;
    else if (!strcmp(argv[1], "add"))
        func = AddCrate;
    else if (!strcmp(argv[1], "del"))
        func = DeleteCrate;
    else if (!strcmp(argv[1], "conn"))
        func = ConnectCrate;
    else if (!strcmp(argv[1], "disc"))
        func = DisconnectCrate;
    else
        {
        oem_printf("  %s\n", argv[1]);
        exit(1);
        }

    /*   TLTR */
    if ((ltr_result = LTR_Init(&ltr)) != LTR_OK)
        {
        oem_printf("    LTR: %s\n", LTR_GetErrorString(ltr_result));
        exit(1);
        }

    /*      */
    oem_printf("\n  ...");
    strncpy(ltr.csn, CSN_SERVER_CONTROL, sizeof(ltr.csn));
    ltr.cc = CC_CONTROL;
    if ((ltr_result = LTR_Open(&ltr)) != LTR_OK)
        {
        oem_printf(" : %s\n", LTR_GetErrorString(ltr_result));
        exit(1);
        }

    oem_printf(" %s ok\n", ltr.csn);

    /*    */
    func(&ltr, ip_addr);

    /*  */
    LTR_Close(&ltr);
    oem_printf("\n \n");
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
